#include <iostream>
#include <limits.h>
#include "bleep.h"

using namespace geogame;
using namespace std;

Bleep *Bleep::m_Singleton=NULL;

Bleep::Bleep() :
m_CurrentSource(0)
{
	m_Device=alcOpenDevice(NULL);
	if(m_Device==NULL)
	{
		cerr<<"Could not open OpenAL device"<<endl;
		return;
	}
	
	int attrlist[]={ALC_FREQUENCY,44100,ALC_INVALID};
	m_Context=alcCreateContext(m_Device,attrlist);
	if(m_Context==NULL) 
	{
		cerr<<"Could not open OpenAL context: "<<alGetString(alcGetError(m_Device))<<endl;
		return;
	}
	
	alcMakeContextCurrent(m_Context);
	
	ALfloat zeroes[] = { 0.0f, 0.0f,  0.0f };
	ALfloat front[]  = { 0.0f, 0.0f,  1.0f, 0.0f, 1.0f, 0.0f };
	alListenerfv(AL_POSITION, zeroes);
	alListenerfv(AL_ORIENTATION, front);
	
	alGenSources(MAX_SOURCES,m_Sources);
	alGenSources(MAX_SOURCES,m_LoopingSources);
}

Bleep::~Bleep()
{
	alcDestroyContext(m_Context);
	alcCloseDevice(m_Device);
}

Bleep *Bleep::Get()
{
	if (!m_Singleton) m_Singleton = new Bleep;
	return m_Singleton;
}

int Bleep::Register(float *data, unsigned int size)
{
	short *buf = new short[size];
	for (unsigned int n=0; n<size; n++)
	{
		buf[n]=(short)(data[n]*SHRT_MAX);
	}
	
	return Register(buf,size);
}	

int Bleep::Register(short *data, unsigned int size)
{
	ALuint buffer;
	alGenBuffers(1, &buffer);
	alBufferData(buffer, AL_FORMAT_MONO16, data, size*sizeof(short), 44100);
	if(alGetError() != AL_NO_ERROR) 
	{
		cerr<<"Could not BufferData"<<endl;
		return 1;
	}
	
	return buffer;
}

int Bleep::Play(int id, float freq, float vol, int loop)
{
	int sid;
	if (loop==0)
	{
		sid=m_Sources[m_CurrentSource];
		m_CurrentSource++;
		if (m_CurrentSource>=MAX_SOURCES) m_CurrentSource=0;
		alSourceStop(sid);
	}
	else
	{
		sid=m_LoopingSources[loop];
		alSourceStop(sid);
	}
	
	ALfloat zeroes[] = { 0.0f, 0.0f,  0.0f };
	alSourcei(sid, AL_BUFFER, id);
	alSourcefv(sid,AL_POSITION,zeroes);
	if (loop) alSourcei(sid, AL_LOOPING, AL_TRUE);
	alSourcef(sid, AL_PITCH, freq/440.f);
	alSourcePlay(sid);
	return sid;
}

void Bleep::Modify(int pid, float freq, float vol)
{
	alSourcef(pid, AL_PITCH, freq/440.f);
}

void Bleep::StopLoop(int loopid)
{
	alSourceStop(m_LoopingSources[loopid]);
}

