#include <math.h>
#include <stdlib.h>
#include "feeble.h"
#include <iostream>

#ifdef WIN32
#define M_PI 3.14152
#endif

using namespace geogame;

unsigned int Feeble::Samplerate = 44100;
float Feeble::TimeLength = 1;

Feeble::Feeble()
{
}

Feeble::~Feeble()
{
}

/////////////////////////////////////////////////////////////////////////////

Feeble::Module::Module() :
m_Buffer(NULL),
m_Size(0),
m_Samplerate(Feeble::Samplerate),
m_TimeLength(Feeble::TimeLength)
{
}

Feeble::Module::~Module() 
{ 
	if(m_Buffer) delete[] m_Buffer; 
}

float Feeble::Module::Get(unsigned int n) 
{ 
	if (n<m_Size) return m_Buffer[n]; 
	else return 0; 
}

float Feeble::Module::Execute()
{
	for (map<string,Module*>::iterator i=m_Inputs.begin(); i!=m_Inputs.end(); i++)
	{
		m_TimeLength = i->second->Execute();
	}
	
	return m_TimeLength;
}

float Feeble::Module::GetInput(const string &name, unsigned int n)
{
	map<string,Module*>::iterator i=m_Inputs.find(name);
	if (i!=m_Inputs.end())
	{
		return i->second->Get(n);
	}	
	return 0;
}

void Feeble::Module::Dump()
{
	for (unsigned int n=0; n<m_Size; n++)
	{
		cerr<<m_Buffer[n]<<endl;
	}
}

/////////////////////////////////////////////////////////////////////////////

float Feeble::Value::Execute()
{
	m_Size=(unsigned int)(m_TimeLength*m_Samplerate);
	return m_TimeLength;
}

/////////////////////////////////////////////////////////////////////////////

float Feeble::Mult::Execute()
{
	Feeble::Module::Execute();
	
	unsigned int samplelength = (unsigned int)(m_TimeLength*m_Samplerate);

	if (m_Size!=samplelength || !m_Buffer)
	{
		if (m_Buffer) delete[] m_Buffer;
		m_Buffer = new float[samplelength];
		m_Size=samplelength;
	}

	for (unsigned int n=0; n<m_Size; n++)
	{
		m_Buffer[n] = GetInput("a",n)*GetInput("b",n);
	}
	
	return m_TimeLength;
}

/////////////////////////////////////////////////////////////////////////////

float Feeble::Add::Execute()
{
	Feeble::Module::Execute();
	
	unsigned int samplelength = (unsigned int)(m_TimeLength*m_Samplerate);

	if (m_Size!=samplelength || !m_Buffer)
	{
		if (m_Buffer) delete[] m_Buffer;
		m_Buffer = new float[samplelength];
		m_Size=samplelength;
	}

	for (unsigned int n=0; n<m_Size; n++)
	{
		m_Buffer[n] = GetInput("a",n)+GetInput("b",n);
	}
	
	return m_TimeLength;
}

/////////////////////////////////////////////////////////////////////////////

Feeble::Generator::Generator(Type t) : 
m_Type(t) 
{
}
	
float Feeble::Generator::Execute()
{
	Feeble::Module::Execute();
	
	unsigned int samplelength = (unsigned int)(m_TimeLength*m_Samplerate);

	if (m_Size!=samplelength || !m_Buffer)
	{
		if (m_Buffer) delete[] m_Buffer;
		m_Buffer = new float[samplelength];
		m_Size=samplelength;
	}

	float position=0;
	float cyclepos=0;
	float dampfactor=20; // samples
	float randsandh=0;
	
	for (unsigned int n=0; n<m_Size; n++)
	{
		float freq = GetInput("freq",n);
		float phase = GetInput("phase",n);
		
		position+=phase+M_PI*freq/m_Samplerate;
		cyclepos=fmod((float)position,(float)M_PI)/M_PI;

		switch (m_Type)
		{
			case Feeble::Generator::SINE:     m_Buffer[n]=sin(position); break;
			case Feeble::Generator::SAWTOOTH: m_Buffer[n]=cyclepos; break;
			case Feeble::Generator::REVSAW:   m_Buffer[n]=1-cyclepos; break;
			case Feeble::Generator::TRIANGLE:  
				if (cyclepos<0.5) m_Buffer[n]=cyclepos*2; 
				else m_Buffer[n]=(1-cyclepos)*2;				
			break;
			case Feeble::Generator::NOISE:   
				// hardly scientific
			 	if ((int)freq==0 || n%(int)freq==0) randsandh=(rand()%1000/500.0f)-1.0f; 
				m_Buffer[n]=randsandh; 
			break;
			case Feeble::Generator::SQUARE: break; 
		}
		
		// dampen begin and end of sample to stop clicking
		if (n<dampfactor) m_Buffer[n]*=n/dampfactor;
		else if (n>m_Size-dampfactor) m_Buffer[n]*=(m_Size-n)/dampfactor;
	}
	
	return m_TimeLength;
}

////////////////////////////////////////////////////////////////////

Feeble::Envelope::Envelope(Type t) :
m_Type(t) 
{
}

float Feeble::Envelope::Execute()
{
	Feeble::Module::Execute();
	
	unsigned int samplelength = (unsigned int)(m_TimeLength*m_Samplerate);

	if (m_Size!=samplelength || !m_Buffer)
	{
		if (m_Buffer) delete[] m_Buffer;
		m_Buffer = new float[samplelength];
		m_Size=samplelength;
	}

	for (unsigned int n=0; n<m_Size; n++)
	{
		float time = n/(float)m_Samplerate;
		float t=n/(float)m_Size;
		
		switch (m_Type)
		{
			case Feeble::Envelope::RAMPUP:   m_Buffer[n]=t; break;
			case Feeble::Envelope::RAMPDOWN: m_Buffer[n]=1-t; break;
			case Feeble::Envelope::HILL: 
				if (t<0.5) m_Buffer[n]=t*2; 
				else m_Buffer[n]=(1-t)*2; 
			break;
			case Feeble::Envelope::VALLEY: 
				if (t<0.5) m_Buffer[n]=(1-t)*2; 
				else m_Buffer[n]=t*2; 
			break;
		}
		
	}
	
	return m_TimeLength;
}
