#include "dada.h"
#include "entity.h"
#include <string>
#include <vector>

#ifndef GEO
#define GEO

namespace geogame {

class Component
{
public:
	Component(const dMatrix &space);
	virtual ~Component() {}

	virtual void Render()=0;
	virtual int NumSides()=0;
	const dMatrix &GetSpace() { return m_Space; }
	dVector GetPosition() { return m_Pos; }
	
private:
	dMatrix m_Space;
	dVector m_Pos; // for convienience
};

class TriangleComponent : public Component
{
public:
	TriangleComponent(const dMatrix &space) : Component(space) {}
	virtual ~TriangleComponent() {}
	virtual void Render();
	virtual int NumSides() { return 3; }	
};

class SquareComponent : public Component
{
public:
	SquareComponent(const dMatrix &space) : Component(space) {}
	virtual ~SquareComponent() {}
	virtual void Render();
	virtual int NumSides() { return 4; }	
};

class PentagonComponent : public Component
{
public:
	PentagonComponent(const dMatrix &space) : Component(space) {}
	virtual ~PentagonComponent() {}
	virtual void Render();
	virtual int NumSides() { return 5; }	
};

/////////////////////////////////////////////////////////////////////////////////

class Geo : public Entity
{
public:
	Geo(int generation, const dMatrix &space);
	virtual ~Geo();
	
	virtual void Update(const list<Entity*> &world, const dVector &playerpos);
	virtual void Render();
	virtual bool CheckCollide(const dVector &pos, float radius, bool damage=false);
	virtual float GetMass() { return m_Mass; }
	virtual const dVector &GetPosition() { return m_Position; }

	void Process();
	void Randomise();
	void Mutate(float rate);
	
	class Genome
	{
	public:
		string Description;
		string Axiom;
		vector<pair<char,string> > Rules;
		int Generation;
		int Score;
		bool operator<(const Genome &other) { return Score<other.Score; }
	};
	
	Genome m_Genome;
	
	static int GeoCount;
	
private:
	void RenderPart(const string &part, int depth);
	string ExtractBranch(const string &ls, unsigned int &pos);
	string GetFragment();
	string MutateHelper(string s, float rate);
	
	static void InitAudio();
	static bool m_InitAudio;
	static int m_ExplosionSound;
	static int m_DeathSound;
	static int m_ShotSound;
	
	bool m_Dirty;

	list<Component*> m_Components;
	vector<dMatrix> m_BirthPos;
	vector<dVector> m_FirePos;

	dMatrix m_Space;
	dVector m_Position;
	
	int m_Age;
	int m_Ticks;
	int m_Rate;
	int m_Energy;
	int m_Lifetime;
	int m_NumTriangles;
	int m_NumSquares;
	bool m_JustRebuilt;
	int m_Mass;
};
}
#endif
