#include <cstdio>
#include "glinc.h"
#include "geogame.h"
#include "key.h"
#include "projectiles.h"
#include "utils.h"
#include "feeble.h"
#include "bleep.h"

using namespace geogame;

static const float MAX_GRAV_DIST=30;
static const float EXP_DAMAGE_MULT=5;
static const float EXP_PUSH_MULT=0.3;
static const int LEVELCOMPLETE_DELAY=500;
static const int LIFELOST_DELAY=500;
static const int ENDGAME_DELAY=500;

GeoGame::GeoGame(int W, int H, char *n) :
m_RotX(0),
m_RotY(0),
m_PosX(0),
m_PosY(0),
m_DisY(0),
m_Width(W),
m_Height(H),
m_Scale(0.5),
m_WeaponRecharge(0),
m_PlayerDir(0,1,0),
m_PlayerVec(0,0,0),
m_FirstTime(true),
m_PlayerHealth(100.0),
m_Level(0),
m_Lives(5),
m_Finished(false),
m_LevelComplete(false),
m_Delay(0),
m_FlashCounter(0),
m_Recharging(0),
m_WeaponLevel(10),
m_CurrentWeapon(0),
m_Frame(0),
m_DisplayTextFrames(-1),
m_PauseMode(false)
{

	// init the audio
	InitAudio();
}

void GeoGame::Init()
{
	m_PressedKeys.clear();
	m_BGSoundID = Bleep::Get()->Play(m_BGSound, 220, 1, 1);
	m_PlayerDir=dVector(0,1,0);
	m_PlayerVec=dVector(0,0,0);
	m_PlayerPos=dVector(0,0,0);
	m_Level=0;
	m_PlayerHealth=100.0;
	m_Lives=5;
	m_Finished=false;
	m_LevelComplete=false;	
	Clear();
	m_Store.clear();
	m_Delay=0;
	m_WeaponLevel=1;
	m_CurrentWeapon=0;
	for (int i=0; i<10; i++) m_WeaponEnergy[i]=1;
	MakeNewLevel();
}

void GeoGame::Clear()
{
	for (list<Entity*>::iterator i=m_World.begin(); i!=m_World.end(); i++)	
	{
		delete *i;
	}
	m_World.clear();
}

void GeoGame::Reshape(int width, int height)
{
	m_Width=width;
	m_Height=height;
	m_FirstTime=true;
}
	
void GeoGame::Handle(unsigned char key, int button, int state, int x, int y) 
{
	if (key && state==0) m_PressedKeys.insert(key); // keydown
	if (key && state==1) m_PressedKeys.erase(key); // keyup	
	m_LastMouseX=x;
	m_LastMouseY=y;
	
	if (key=='q') 
	{
		m_Finished=true;
		Clear();
		Bleep::Get()->StopLoop(1);
	}
	
}

void GeoGame::Update()
{ 
	if (m_PressedKeys.find('p')!=m_PressedKeys.end()) 
	{
		m_PressedKeys.clear();
		m_PauseMode=!m_PauseMode;
	}
	if (m_PauseMode) return;
	
	if (m_PressedKeys.find(GLUT_KEY_LEFT)!=m_PressedKeys.end()) 
	{
		dMatrix m;
		m.rotz(5);
		m_PlayerDir=m.transform(m_PlayerDir);
	}
	
	if (m_PressedKeys.find(GLUT_KEY_RIGHT)!=m_PressedKeys.end()) 
	{
		dMatrix m;
		m.rotz(-5);
		m_PlayerDir=m.transform(m_PlayerDir);
	}
	
	if (m_PressedKeys.find(GLUT_KEY_UP)!=m_PressedKeys.end()) 
	{
		m_PlayerVec+=m_PlayerDir*0.007;
		dVector dir(-m_PlayerDir.x*RandRange(0,0.02),
					-m_PlayerDir.y*RandRange(0,0.02),
					-m_PlayerDir.z*RandRange(0,0.02));			
		if (m_ThrustParticles) m_ThrustParticles->AddParticle(Particles::Particle(m_PlayerPos,dir,m_PlayerVec));
	}
	
	if (m_PressedKeys.find(GLUT_KEY_DOWN)!=m_PressedKeys.end()) 
	{
		m_PlayerVec-=m_PlayerDir*0.007;
	}
		
	if (m_PressedKeys.find(' ')!=m_PressedKeys.end()) 
	{
		if (m_WeaponEnergy>0)
		{
			m_WeaponEnergy[m_CurrentWeapon]-=0.005;
			switch (m_CurrentWeapon)
			{
				case 1:
					if (m_WeaponRecharge*m_WeaponEnergy[1]>4)
					{
						m_WeaponRecharge=0;
						Bleep::Get()->Play(m_FireSound, 220, 1);
						m_World.push_back(new Bullet(m_PlayerPos,m_PlayerDir*0.4,m_PlayerVec));
					}
				break;

				case 2:
				{
					if (m_WeaponRecharge*m_WeaponEnergy[2]>2)
					{
						m_WeaponRecharge=0;
						Bleep::Get()->Play(m_FireSound, 220, 1);
						float angle=sin(m_Frame*0.2)*0.2;
						if (m_PlayerDir.y>0) angle+=atan(m_PlayerDir.x/m_PlayerDir.y);
						else angle+=atan(m_PlayerDir.x/m_PlayerDir.y)+3.141; // oh, this is nice...
						dVector dir(sin(angle),cos(angle),0);
						m_World.push_back(new Bullet(m_PlayerPos,dir*0.25,m_PlayerVec));
					}
				}
				break;

				case 3:
					if (m_WeaponRecharge*m_WeaponEnergy[3]>10)
					{	
						m_WeaponRecharge=0;
						Bleep::Get()->Play(m_FireSound, 220, 1);
						Missile *newmissile=new Missile(m_PlayerPos,m_PlayerDir*0.4,m_PlayerVec);
						newmissile->SetExplodeSound(m_ExplosionSound);
						m_World.push_back(newmissile);
					}
				break;

				case 4:
				{
					if (m_WeaponRecharge*m_WeaponEnergy[4]>0.5)
					{
						m_WeaponEnergy[4]-=0.02;
						m_WeaponRecharge=0;
						Bleep::Get()->Play(m_FireSound, 220, 1);
						float angle=m_Frame*0.2;
						if (m_PlayerDir.y>0) angle+=atan(m_PlayerDir.x/m_PlayerDir.y);
						else angle+=atan(m_PlayerDir.x/m_PlayerDir.y)+3.141; // oh, this is nice...
						dVector dir(sin(angle),cos(angle),0);
						m_World.push_back(new Bullet(m_PlayerPos,dir*0.25,m_PlayerVec));
					}
				}
				break;

				case 5:
					if (m_WeaponRecharge*m_WeaponEnergy[5]>5)
					{	
						m_WeaponRecharge=0;
						Bleep::Get()->Play(m_FireSound, 220, 1);
						float angle=sin(m_Frame*0.2)*0.3;
						if (m_PlayerDir.y>0) angle+=atan(m_PlayerDir.x/m_PlayerDir.y);
						else angle+=atan(m_PlayerDir.x/m_PlayerDir.y)+3.141; // oh, this is nice...
						dVector dir(sin(angle),cos(angle),0);
						Missile *newmissile=new Missile(m_PlayerPos,dir*0.25,m_PlayerVec);
						newmissile->SetExplodeSound(m_ExplosionSound);
						m_World.push_back(newmissile);
					}
				break;

				case 6:
					if (m_WeaponRecharge*m_WeaponEnergy[6]>20)
					{	
						m_WeaponRecharge=0;
						Mutator *newmissile=new Mutator(m_PlayerPos,m_PlayerDir*0.1,m_PlayerVec);
						newmissile->SetExplodeSound(m_MuteSound);
						m_World.push_back(newmissile);
					}
				break;
			}
		}
	}
	
	int selectedweapon=99;
	if (m_PressedKeys.find('1')!=m_PressedKeys.end()) selectedweapon=1;
	if (m_PressedKeys.find('2')!=m_PressedKeys.end()) selectedweapon=2;
	if (m_PressedKeys.find('3')!=m_PressedKeys.end()) selectedweapon=3;
	if (m_PressedKeys.find('4')!=m_PressedKeys.end()) selectedweapon=4;
	if (m_PressedKeys.find('5')!=m_PressedKeys.end()) selectedweapon=5;
	if (m_PressedKeys.find('6')!=m_PressedKeys.end()) selectedweapon=6;
	if (m_PressedKeys.find('7')!=m_PressedKeys.end()) selectedweapon=7;
	if (m_PressedKeys.find('8')!=m_PressedKeys.end()) selectedweapon=8;
	if (m_PressedKeys.find('9')!=m_PressedKeys.end()) selectedweapon=9;
	
	for (int i=0; i<10; i++) 
	{
		m_WeaponEnergy[i]+=0.003;
		if (m_WeaponEnergy[i]>1) m_WeaponEnergy[i]=1;
	}
	
	m_WeaponRecharge++;
	if (selectedweapon<=m_WeaponLevel) m_CurrentWeapon=selectedweapon;
	
	if (m_PressedKeys.find('=')!=m_PressedKeys.end()) 
	{
		m_Scale*=1.01;
	}
	
	if (m_PressedKeys.find('-')!=m_PressedKeys.end()) 
	{
		m_Scale*=0.99;
	}
	
	if (m_PressedKeys.find('c')!=m_PressedKeys.end()) 
	{
		for (list<Entity*>::iterator i=m_World.begin(); i!=m_World.end(); i++)
		{
			if ((*i)->GetFlags()&ENTITYFLAG_ENEMY && (*i)->GetPosition().dist(m_PlayerPos)<10)
			{
				Geo* geo=(Geo*)*i;
				geo->Mutate(0.9);
			}
		}
	}
	
	dVector gravity;
	list<Entity*> newentities;
	vector<list<Entity*>::iterator> dellist;
	for (list<Entity*>::iterator i=m_World.begin(); i!=m_World.end(); i++)
	{
		(*i)->Update(m_World,m_PlayerPos);
		list<Entity*> temp = (*i)->GetNewEntities();
		newentities.merge(temp);
		if ((*i)->RemoveMe()) dellist.push_back(i);
		
		// contribute to player gravity
		if ((*i)->GetMass())
		{	
			dVector bet=(*i)->GetPosition()-m_PlayerPos;
			float distance = bet.mag();
			bet.normalise();
			if (distance<MAX_GRAV_DIST)
			{
				// newton innit? :)
				gravity+=bet*(1-(distance/MAX_GRAV_DIST))*((*i)->GetMass()*0.000005);
			}
		}
	}
	
	// update player pos
	m_PlayerVec+=gravity;
	m_PlayerPos+=m_PlayerVec;
	
	if (m_PlayerPos.x<-200) m_PlayerPos.x+=400;
	if (m_PlayerPos.x>200) m_PlayerPos.x-=400;
	if (m_PlayerPos.y<-200) m_PlayerPos.y+=400;
	if (m_PlayerPos.y>200) m_PlayerPos.y-=400;
		
	for (vector<list<Entity*>::iterator>::iterator i=dellist.begin(); i!=dellist.end(); i++)
	{
		list<Entity*>::iterator e=*i;
		if ((*e)->GetFlags()==ENTITYFLAG_ENEMY)
		{
			Geo *g=(Geo*)*e;
			// keep the genome if it's scored
			if (g->m_Genome.Score>0)
			{
				m_Store.push_back(g->m_Genome);
			}
		}
		
		if ((*e)->GetFlags()&ENTITYFLAG_CAUGHT_WEAPONPICKUP)
		{
			m_WeaponLevel++;
			char buf[10];
			sprintf(buf,"%d",m_WeaponLevel);
			DisplayText("WEAPON LEVEL NOW AT "+string(buf),80);
		}
		
		delete *e;
		m_World.erase(*i);
	}
	
	m_World.merge(newentities);
	
	if (m_LevelComplete) 
	{
		m_Delay++;
		if (m_Delay>LEVELCOMPLETE_DELAY)
		{
			MakeNewLevel();
		}
		
	}
	
	if (!m_World.empty()) m_LevelComplete=true;

	m_Recharging=false;

	// player collision
	for (list<Entity*>::iterator g=m_World.begin(); g!=m_World.end(); g++)
	{
		Entity *other = *g;
		if (m_PlayerHealth!=0 && other->GetFlags()==ENTITYFLAG_ENEMY)
		{
			if (other->CheckCollide(m_PlayerPos,0)) 
			{
				// landing on enemy increases health...
				m_PlayerHealth+=0.1;
				m_Recharging=true;
				m_PlayerVec=-m_PlayerVec/2.0;
			}
		}
		
		if (other->GetFlags()==ENTITYFLAG_KEY)
		{
			m_LevelComplete=false;
		}

		else if (other->GetFlags()==ENTITYFLAG_EXPLOSION)
		{
			dVector dist=m_PlayerPos-other->GetPosition();
			float d=dist.mag();
			if (d<1)
			{
				m_PlayerHealth-=(int)(EXP_DAMAGE_MULT*(1-d));
				m_PlayerVec+=(dist*(1-d)*EXP_PUSH_MULT);
			}
		}
	}
	
	if (m_LevelComplete) DisplayText("LEVEL COMPLETE!",100);

	
	if (m_PlayerHealth<=0) 
	{
		m_PlayerHealth=0;
		m_Delay++;
		
		if (m_Delay>LIFELOST_DELAY)
		{
			m_PlayerHealth=100;
			m_Lives--;
			if (m_Lives<0)
			{
				DisplayText("GAME OVER",100);
				m_Finished=true;
				Bleep::Get()->StopLoop(1);
			}
			m_Delay=0;
		}
	}
	
	if (m_PlayerHealth>100) m_PlayerHealth=100;	
	
	Bleep::Get()->Modify(m_BGSoundID, Geo::GeoCount, 1);
	
	m_Frame++;
}

void GeoGame::RenderPlayer()
{
	//glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	if (m_Recharging) 
	{
		m_FlashCounter++;
		if (m_FlashCounter>50) m_FlashCounter=0;
		glColor3f(m_FlashCounter/50.0f,0,1);
	}
	else glColor3f(1,1,1);
	glPushMatrix();
	glTranslatef(m_PlayerPos.x,m_PlayerPos.y,0);
	glScalef(0.5,0.5,0.5);
	glBegin(GL_TRIANGLES);
	glVertex3f(m_PlayerDir.x,m_PlayerDir.y,0);
	dVector cross=m_PlayerDir.cross(dVector(0,0,1));
	dVector v1 = cross-m_PlayerDir;
	dVector v2 = (-cross)-m_PlayerDir;
	glVertex3f(v1.x*0.3,v1.y*0.3,0);
	glVertex3f(v2.x*0.3,v2.y*0.3,0);
	glEnd();
	glPopMatrix();
	//glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
}

void GeoGame::RenderHUD()
{
	glLoadIdentity();		
    glColor4f(1,1,1,1);
    glTranslatef(2.0,-6.5,0);
	char text[256];
	sprintf(text,"Population: %d Level:%d Lives:%d Health: %d",Geo::GeoCount,m_Level,m_Lives,(int)m_PlayerHealth);
	DrawText(text);
}

void GeoGame::Render()
{	
	if (m_FirstTime)
	{
		glViewport(0,0,m_Width,m_Height);
		glMatrixMode (GL_PROJECTION);
  		glLoadIdentity();
		//glOrtho(-1,1,-0.7,0.7,0.1,100);
		glOrtho(-10,10,-7,7,0.1,100);
		glEnable(GL_BLEND);
    	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_LIGHTING);
		glEnable(GL_COLOR_MATERIAL);
		m_FirstTime=false;
	}
		
	glClear(GL_COLOR_BUFFER_BIT);
	glMatrixMode (GL_MODELVIEW);
	
	// render the scanner
	glLoadIdentity();
	//glScalef(m_Scale,m_Scale,m_Scale);
	glScalef(0.05,0.05,0.05);
	glColor3f(0,0.5,0);
	glTranslatef(-m_PlayerPos.x,-m_PlayerPos.y,-10);	
	//if(m_PlayerDir.x) glRotatef(-atan(m_PlayerDir.y/m_PlayerDir.x)*RAD_CONV,0,0,1);
	for(list<Entity*>::iterator i=m_World.begin(); i!=m_World.end(); i++)
	{			
		// scanner detects enemies
		if ((*i)->GetFlags()==ENTITYFLAG_ENEMY)
		{
			
			glPointSize(2);
			glBegin(GL_POINTS);
			glVertex3f((*i)->GetPosition().x,(*i)->GetPosition().y,0);
			glEnd();
		}
		
		// scanner detects keys
		if ((*i)->GetFlags()==ENTITYFLAG_KEY)
		{
			glPushMatrix();
			glTranslatef((*i)->GetPosition().x,(*i)->GetPosition().y,0);
			Primitive::Get()->ScannerKey();
			glPopMatrix();
		}
	}
	glPointSize(1);		
	glLoadIdentity();
	glScalef(m_Scale,m_Scale,m_Scale);
	//if(m_PlayerDir.x) glRotatef(-atan(m_PlayerDir.y/m_PlayerDir.x)*RAD_CONV,0,0,1);
	glTranslatef(-m_PlayerPos.x,-m_PlayerPos.y,-10);
	
	// render the world
	for(list<Entity*>::iterator i=m_World.begin(); i!=m_World.end(); i++)
	{
		if ((*i)->GetFlags()==ENTITYFLAG_PARTICLES || m_PlayerPos.distsq((*i)->GetPosition())<1000)
		{
			(*i)->Render();
		}
	}
	
	if (m_PlayerHealth!=0) 
	{
		RenderPlayer();
	}
	else // make player flash if life lost
	{
		if (m_Flash) RenderPlayer();
		if (m_FlashCounter>5) 
		{
			m_FlashCounter=0;
			m_Flash=!m_Flash;
		}
		m_FlashCounter++;
	}	
	
	RenderHUD();
	RenderText();
}

void GeoGame::DisplayText(string text, int frames)
{
	m_DisplayText=text;
	m_DisplayTextFrames=frames;
}

void GeoGame::RenderText()
{
	if (m_DisplayTextFrames>0)
	{	
		glLoadIdentity();		
    	glColor4f(1,1,1,1);
    	glTranslatef(2.0,0,0);
		DrawText(m_DisplayText);
	}
	m_DisplayTextFrames--;
}


  
bool GeoGame::Finished() 
{ 
	bool temp=m_Finished;
	m_Finished=false;
	return temp; 
}

void GeoGame::MakeNewLevel()
{
	Clear();
	//m_Store.sort();
	Bleep::Get()->Play(m_SpawnSound, 220, 1);
	m_Level++;
	m_Lives++;
	m_PlayerVec=dVector(0,0,0);
	m_Delay=0;
	
	if (!m_Store.empty()) // get rid of genomes with score less than the average...
	{
		int Average=0;
		for (vector<Geo::Genome>::iterator i=m_Store.begin(); i!=m_Store.end(); i++)
		{
			Average+=i->Score;
		}
		
		Average/=m_Store.size();
		vector<Geo::Genome> newstore;
		
		for (vector<Geo::Genome>::iterator i=m_Store.begin(); i!=m_Store.end(); i++)
		{
			if (i->Score>=Average) newstore.push_back(*i);
		}
		
		m_Store=newstore;
	}
	
	m_ThrustParticles = new Particles(100);
	m_World.push_back(m_ThrustParticles);

	
	float worldsize=m_Level*5;
	if (worldsize>50) worldsize=50;
	
	int numgeos=m_Level*20;
	if (numgeos>200) numgeos=200;

	m_World.push_back(new WeaponPickup(dVector(RandRange(-worldsize*4,worldsize*4),RandRange(-worldsize*4,worldsize*4),0),0.1));
	
	for (int n=0; n<numgeos; n++)
	{
		dMatrix m;
		m.scale(3,3,3);
		m.translate(RandRange(-worldsize,worldsize),RandRange(-worldsize,worldsize),0);
		Geo* newgeo=new Geo(0,m);
		if (n<numgeos/2 || m_Store.empty()) 
		{
			newgeo->Randomise();
		}
		else 
		{
			newgeo->m_Genome=m_Store[rand()%m_Store.size()]; // bias to top?
			newgeo->Mutate(0.2);
		}
		m_World.push_back(newgeo);
	}

	int numkeys=m_Level+2;
	if (numkeys>20) numkeys=20;
	
	for (int n=0; n<numkeys; n++)
	{
		dVector p(RandRange(-worldsize*4,worldsize*4),RandRange(-worldsize*4,worldsize*4),0);
		Key* newkey=new Key(p,0.05+n/10.0f);
		newkey->SetCaptureSound(m_CaptureSound);
		m_World.push_back(newkey);
	}
}

void GeoGame::InitAudio()
{
	Feeble::TimeLength=5;
	Feeble::Samplerate=22050;
	
	Feeble::TimeLength=2;
	Feeble::Value volvalue(0.1);
	Feeble::Mult volmult;

	// spawn sound
	{
	Feeble::Envelope env(Feeble::Envelope::HILL);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(1000);
	Feeble::Generator master(Feeble::Generator::SINE);
	Feeble::Value multvalue(4000);
	Feeble::Add add;
	Feeble::Value addvalue(10);
	Feeble::Mult mult;
	Feeble::Generator slave(Feeble::Generator::SINE);
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	master.Patch("freq",&envmult);
	mult.Patch("a",&master);
	mult.Patch("b",&multvalue);
	add.Patch("a",&mult);
	add.Patch("b",&addvalue);
	slave.Patch("freq",&add);
	volmult.Patch("a",&slave);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_SpawnSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
	
	// shot sound
	{
	Feeble::TimeLength=0.1;
	Feeble::Envelope env(Feeble::Envelope::RAMPDOWN);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(5000);
	Feeble::Generator osc(Feeble::Generator::SINE);	
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	osc.Patch("freq",&envmult);
	volmult.Patch("a",&osc);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_FireSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
	
	// capture sound
	{
	Feeble::TimeLength=0.1;
	Feeble::Value modfreq(2);
	Feeble::Generator mod(Feeble::Generator::SINE);
	Feeble::Value modamount(2);
	Feeble::Value modaddvalue(220);
	Feeble::Mult mult;
	Feeble::Add modadd;
	Feeble::Generator osc1(Feeble::Generator::SAWTOOTH);
	Feeble::Value freq2(220);
	Feeble::Generator osc2(Feeble::Generator::SAWTOOTH);
	Feeble::Add add;
	
	mod.Patch("freq",&modfreq);
	mult.Patch("a",&mod);
	mult.Patch("b",&modamount);	
	modadd.Patch("a",&mult);
	modadd.Patch("b",&modaddvalue);
	osc1.Patch("freq",&modadd);
	osc2.Patch("freq",&freq2);
	add.Patch("a",&osc1);
	add.Patch("b",&osc2);
	volmult.Patch("a",&add);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
	
	m_CaptureSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}

	// player explosion sound
	{
	Feeble::TimeLength=0.5;
	Feeble::Envelope env(Feeble::Envelope::RAMPUP);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(50);
	Feeble::Generator osc(Feeble::Generator::NOISE);	
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	osc.Patch("freq",&envmult);
	volmult.Patch("a",&osc);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_ExplosionSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
	
	// mutate sound
	{
	Feeble::Envelope env(Feeble::Envelope::RAMPDOWN);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(300);
	Feeble::Generator master(Feeble::Generator::SINE);
	Feeble::Value multvalue(40000);
	Feeble::Add add;
	Feeble::Value addvalue(440);
	Feeble::Mult mult;
	Feeble::Generator slave(Feeble::Generator::SAWTOOTH);
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	master.Patch("freq",&envmult);
	mult.Patch("a",&master);
	mult.Patch("b",&multvalue);
	add.Patch("a",&mult);
	add.Patch("b",&addvalue);
	slave.Patch("freq",&add);
	volmult.Patch("a",&slave);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_MuteSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
	
	// bg sound
	{
	Feeble::TimeLength=2;
	Feeble::Envelope env(Feeble::Envelope::RAMPDOWN);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(100);
	Feeble::Generator master(Feeble::Generator::SINE);
	Feeble::Value multvalue(100);
	Feeble::Add add;
	Feeble::Value addvalue(40);
	Feeble::Mult mult;
	Feeble::Generator slave(Feeble::Generator::SINE);
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	master.Patch("freq",&envmult);
	mult.Patch("a",&master);
	mult.Patch("b",&multvalue);
	add.Patch("a",&mult);
	add.Patch("b",&addvalue);
	slave.Patch("freq",&add);
	volmult.Patch("a",&slave);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_BGSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
}
