#include <sys/time.h>
#include <list>
#include <set>
#include "geo.h"
#include "projectiles.h"
#include "particles.h"

#ifndef GEOGAME 
#define GEOGAME

namespace geogame {

class GeoGame
{
public:
	GeoGame(int w, int h, char *n);
	virtual ~GeoGame() {}
	
	void Init();
	void Handle(unsigned char key, int button, int state, int x, int y);
	void Update();
	void Reshape(int width, int height);
	void Render();
	bool Finished();
	void AddEntity(Entity *e) { m_World.push_back(e); }
	
private:
	void RenderPlayer();
	void RenderHUD();
	void InitAudio();
	void Clear();
	void MakeNewLevel();
	void DisplayText(string text, int frames);
	void RenderText();
	
	int m_LastMouseX;
	int m_LastMouseY;
	float m_RotX,m_RotY,m_PosX,m_PosY,m_DisY;
	bool m_DisplayGenes;
	set<int> m_PressedKeys;
	
	list<Entity*> m_World;
	vector<Geo::Genome> m_Store;
	
	Particles *m_ThrustParticles;
	
	int m_Width,m_Height;
	float m_Scale;
	
	int m_WeaponRecharge;
	
	dVector m_PlayerPos;
	dVector m_PlayerDir;
	dVector m_PlayerVec;
	
	bool m_FirstTime;
	float m_PlayerHealth;
	int m_Level;
	int m_Lives;
	bool m_Finished;
	bool m_LevelComplete;
	int m_Delay;
	bool m_LifeLost;
	bool m_Flash;
	int m_FlashCounter;
	bool m_Recharging;
	int m_WeaponLevel;
	int m_CurrentWeapon;
	unsigned int m_Frame;
	float m_WeaponEnergy[10];
	
	string m_DisplayText;
	int m_DisplayTextFrames;
	bool m_PauseMode;
	
	int m_SpawnSound;
	int m_FireSound;
	int m_GeoExplosionSound;
	int m_ExplosionSound;
	int m_CaptureSound;
	int m_MuteSound;
	int m_BGSound;
	
	int m_BGSoundID;
};
}
#endif
