#include "glinc.h"
#include <iostream>
#include "intro.h"
#include "utils.h"
#include "feeble.h"
#include "bleep.h"

using namespace geogame;

bool Intro::m_InitAudio=false;
int Intro::m_MenuSound;
int Intro::m_SelectSound;
int Intro::m_BGSound;
int Intro::m_BG2Sound;

Intro::Intro(int width, int height, GeoGame *game):
m_Width(width),
m_Height(height),
m_CurrentScreen(NULL),
m_Game(game),
m_Finished(false),
m_FirstTime(true)
{
	if (!m_InitAudio)
	{
		InitAudio();
	}		
	
	Bleep::Get()->Play(m_BGSound, 220, 1, 1);
	Bleep::Get()->Play(m_BGSound, 330, 1, 2);
	Bleep::Get()->Play(m_BG2Sound, 220, 1, 3);
	
	for (int n=0; n<10; n++)
	{
		dMatrix m;
		m.translate(RandRange(-2,2),RandRange(-2,2),-10);
		m_Geos.push_back(new Geo(0,m));
	}
			
	for (list<Geo*>::iterator i=m_Geos.begin(); i!=m_Geos.end(); i++)
	{
		(*i)->Randomise();
		for (int n=0; n<100; n++) (*i)->Process();
		list<Entity*> world;
		(*i)->Update(world,dVector(0,0,0));
	}
}

Intro::~Intro() 
{
}

void Intro::Init()
{
	Bleep::Get()->Play(m_BGSound, 220, 1, 1);
	Bleep::Get()->Play(m_BGSound, 330, 1, 2);
	Bleep::Get()->Play(m_BG2Sound, 220, 1, 3);
}

void Intro::Build(const string &screen, const string &desc)
{
	Screen* newscreen = new Screen(m_Game);
    #ifdef WIN32
    istringstream stream(desc.c_str());
    #else
	istringstream stream(desc);
    #endif
	newscreen->Build(stream);
	m_Screens[screen]=newscreen;
	m_CurrentScreen=newscreen;
}

void Intro::Handle(unsigned char key, int button, int state, int x, int y)
{
	m_CurrentScreen->Handle(key,button,state,x,y);
}

void Intro::Update()
{

	m_CurrentScreen->Update();
	string token=m_CurrentScreen->GetToken();
	if (token!="")
	{
		// see if it's asked to switch to a new screen
		map<string,Screen*>::iterator i=m_Screens.find(token);
		if (i!=m_Screens.end())
		{
			m_CurrentScreen=i->second;
		}

		for (list<Geo*>::iterator i=m_Geos.begin(); i!=m_Geos.end(); i++)
		{
			(*i)->Randomise();
			for (int n=0; n<100; n++) (*i)->Process();
			list<Entity*> world;
			(*i)->Update(world,dVector(0,0,0));
		}
		
		if (token=="exit")
		{
			exit(0);
		}
		
		if (token=="finished")
		{
			m_Finished=true;
			Bleep::Get()->StopLoop(1);
			Bleep::Get()->StopLoop(2);
			Bleep::Get()->StopLoop(3);
		}
	}
}

void Intro::Reshape(int width, int height)
{
	m_Width=width;
	m_Height=height;
	m_FirstTime=true;
}

void Intro::Render()
{	
	if (m_FirstTime)
	{
		glViewport(0,0,m_Width,m_Height);
		glMatrixMode (GL_PROJECTION);
  		glLoadIdentity();
		//glOrtho(-1,1,-0.7,0.7,0.1,100);
		glOrtho(-10,10,-7,7,0.1,100);
		glEnable(GL_BLEND);
    	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_LIGHTING);
		glEnable(GL_COLOR_MATERIAL);
		m_FirstTime=false;
	}
	
	glClear(GL_COLOR_BUFFER_BIT);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();
	
	glPushMatrix();
	glScalef(5,5,5);
	// render the bg geos
	for (list<Geo*>::iterator i=m_Geos.begin(); i!=m_Geos.end(); i++)
	{
		(*i)->Render();
	}
	glPopMatrix();

	// render the screen	
	m_CurrentScreen->Render();
}

bool Intro::Finished() 
{ 
	bool temp=m_Finished;
	m_Finished=false;
	return temp; 
}

void Intro::InitAudio()
{
	Feeble::Samplerate=22050;	
	Feeble::TimeLength=5;
	Feeble::Value volvalue(0.1);
	Feeble::Mult volmult;

	// menu sound
	{
	Feeble::TimeLength=0.1;
	Feeble::Envelope env(Feeble::Envelope::RAMPUP);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(3000);
	Feeble::Generator osc(Feeble::Generator::SINE);	
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	osc.Patch("freq",&envmult);
	volmult.Patch("a",&osc);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_MenuSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
	
	// menu sound
	{
	Feeble::TimeLength=0.1;
	Feeble::Envelope env(Feeble::Envelope::RAMPDOWN);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(6000);
	Feeble::Generator osc(Feeble::Generator::SINE);	
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	osc.Patch("freq",&envmult);
	volmult.Patch("a",&osc);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_SelectSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
	
	// bg sound
	{
	Feeble::TimeLength=5;
	Feeble::Envelope env(Feeble::Envelope::HILL);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(100);
	Feeble::Generator master(Feeble::Generator::SINE);
	Feeble::Value multvalue(400);
	Feeble::Add add;
	Feeble::Value addvalue(10);
	Feeble::Mult mult;
	Feeble::Generator slave(Feeble::Generator::SINE);
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	master.Patch("freq",&envmult);
	mult.Patch("a",&master);
	mult.Patch("b",&multvalue);
	add.Patch("a",&mult);
	add.Patch("b",&addvalue);
	slave.Patch("freq",&add);
	volmult.Patch("a",&slave);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_BGSound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}
	
	// bg2 sound
	{
	Feeble::TimeLength=2;
	Feeble::Envelope env(Feeble::Envelope::RAMPDOWN);
	Feeble::Mult envmult;
	Feeble::Value envmultvalue(1000);
	Feeble::Generator master(Feeble::Generator::SINE);
	Feeble::Value multvalue(100);
	Feeble::Add add;
	Feeble::Value addvalue(400);
	Feeble::Mult mult;
	Feeble::Generator slave(Feeble::Generator::SINE);
	
	envmult.Patch("a",&envmultvalue);
	envmult.Patch("b",&env);
	master.Patch("freq",&envmult);
	mult.Patch("a",&master);
	mult.Patch("b",&multvalue);
	add.Patch("a",&mult);
	add.Patch("b",&addvalue);
	slave.Patch("freq",&add);
	volmult.Patch("a",&slave);
	volmult.Patch("b",&volvalue);
	volmult.Execute();
		
	m_BG2Sound = Bleep::Get()->Register(volmult.GetBuffer(),volmult.GetSize());
	}

}

/////////////////////////////////////////////////////////////////////

Screen::Screen(GeoGame *game) : 
m_Game(game), 
m_Finished(false)
{
}

void Screen::Handle(unsigned char key, int button, int state, int x, int y)
{
	for (list<Screen::Widget*>::iterator i=m_Widgets.begin(); i!=m_Widgets.end(); i++)
	{
		if ((*i)->Handle(key,button,state,x,y))
		{ 
			if ((*i)->Activated()) 
			{
				m_Token=(*i)->GetToken();
			}
			return;
		}
	}
}

void Screen::Update()
{
	for (list<Screen::Widget*>::iterator i=m_Widgets.begin(); i!=m_Widgets.end(); i++)
	{
		(*i)->Update();
	}
}

void Screen::Render()
{	
	glColor3f(1,1,1);
	glPushMatrix();
	glTranslatef(-6,5,0);
	glScalef(3,3,3);
	glLineWidth(3);
	DrawText(m_Title);
	glLineWidth(1);
	glPopMatrix();

	for (list<Screen::Widget*>::iterator i=m_Widgets.begin(); i!=m_Widgets.end(); i++)
	{
		(*i)->Render();
	}

}

bool Screen::Result(string& result) 
{ 
	result=m_Token;
	m_Token="";
	bool temp=m_Finished;
	m_Finished=false;
	return temp; 
}

string GetString(istringstream &s)
{
	// ignore space and [
	s.ignore(2);
	char buf[4096];
	s.getline(buf,4096,']');
	return buf;
}

void Screen::Build(istringstream &s)
{
	Button* firstbutton=NULL;
	Button* lastbutton=NULL;
	float ypos=0;
	while (!s.eof())
	{
		// get type token
		string type;
		s>>type;

		if (type=="Title")
		{
			m_Title=GetString(s);
		}
		else if (type=="Button")
		{	
			string name=GetString(s);
			string token=GetString(s);
			Button* newbutton=new Button(name,token,0,ypos++);
			if (!lastbutton) firstbutton=newbutton;
			else 
			{
				newbutton->Previous(lastbutton);
				lastbutton->Next(newbutton);
			}
			lastbutton=newbutton;
			m_Widgets.push_back(newbutton);
		}
		else if (type=="Text")
		{
			string text=GetString(s);
			m_Widgets.push_back(new Text(text,0,ypos));
			ypos+=0.5;
		}	
		else if (type=="Gap")
		{
			ypos+=0.5;
		}	
	}
	
	if (firstbutton)
	{
		firstbutton->Previous(lastbutton);
		lastbutton->Next(firstbutton);
		firstbutton->Select();
	}
}

/////////////////////////////////////////////////////////////////

Screen::Button::Button(const string &name, const string &token, float x, float y) :
m_Name(name),
m_X(x),
m_Y(y),
m_Selected(false),
m_Activated(false),
m_Previous(NULL),
m_Next(NULL),
m_Token(token)
{
}

bool Screen::Button::Activated()
{ 
	bool temp=m_Activated;
	m_Activated=false;
	return temp; 
}

bool Screen::Button::Handle(unsigned char key, int button, int state, int x, int y)
{
	if (m_Selected && state==0)
	{
		if (key==GLUT_KEY_UP && m_Previous!=NULL)
		{
			Bleep::Get()->Play(Intro::m_MenuSound, 220, 1);
			m_Selected=false;
			m_Previous->Select();
			return true;
		}
		
		if (key==GLUT_KEY_DOWN && m_Next!=NULL)
		{
			Bleep::Get()->Play(Intro::m_MenuSound, 220, 1);
			m_Selected=false;
			m_Next->Select();
			return true;
		}
		
		if (key==13)
		{
			Bleep::Get()->Play(Intro::m_SelectSound, 220, 1);
			m_Activated=true;
			return true;
		}
	}
	
	return false;
}

void Screen::Button::Update()
{
}

void Screen::Button::Render()
{
	if (m_Selected) 
	{
		glColor4f(1,1,1,0.4);
		for (int n=0; n<3; n++)
		{
			glPushMatrix();
			glScalef(RandRange(0.99,1.01),RandRange(0.99,1.01),1);
			glTranslatef(m_X-5,-m_Y+3,0);
			DrawText(m_Name);
			glPopMatrix();
		}
	}
	else 
	{
		glColor3f(1,1,1);
		glPushMatrix();
		glTranslatef(m_X-5,-m_Y+3,0);
		DrawText(m_Name);
		glPopMatrix();
	}
}

/////////////////////////////////////////////////////////////////////////////

Screen::Text::Text(const string &text, float x, float y) :
m_X(x),
m_Y(y),
m_Text(text)
{
}

void Screen::Text::Render()
{
	glColor3f(1,1,1);
	glPushMatrix();
	glTranslatef(m_X-5,-m_Y+3,0);
	DrawText(m_Text);
	glPopMatrix();
}
