#include <list>
#include <map>
#ifdef WIN32
#include <strstream>
#define istringstream istrstream
#else
#include <sstream>
#endif
#include "geogame.h"
#include "geo.h"

using namespace std;

namespace geogame {

class Screen;

class Intro
{
public:
	Intro(int width, int Height, GeoGame *game);
	virtual ~Intro();
	
	void Init();
	void Build(const string &screen, const string &desc);
	void Handle(unsigned char key, int button, int state, int x, int y);
	void Update();
	void Reshape(int width, int height);
	void Render();
	bool Finished();
	
	static int m_MenuSound;
	static int m_SelectSound;
	static int m_BGSound;
	static int m_BG2Sound;
	
private:
	void InitAudio();
	static bool m_InitAudio;

	list<Geo*> m_Geos;

	int m_Width,m_Height;
	map<string,Screen*> m_Screens;
	Screen* m_CurrentScreen;
	GeoGame *m_Game;
	bool m_Finished;
	bool m_FirstTime;
};

class Screen
{
public:
	Screen(GeoGame *game);
	virtual ~Screen() {}
	
	virtual void Handle(unsigned char key, int button, int state, int x, int y);
	virtual void Update();
	virtual void Render();
	virtual bool Result(string& result);	
	void SetTitle(const string &s) { m_Title=s; }
	void Build(istringstream &s);
	string GetToken() { string t=m_Token; m_Token=""; return t; }
	
	
	class Widget
	{
	public:
		Widget() {}
		virtual bool Activated()=0;
		virtual bool Handle(unsigned char key, int button, int state, int x, int y)=0;
	    virtual void Update()=0;
		virtual void Render()=0;
		virtual string GetToken() { return ""; }
		
	private:
		
	};
	
	class Button : public Widget
	{
	public:
		Button(const string &name, const string &token, float x, float y);
		virtual ~Button() {}
		
		virtual bool Activated();
		virtual bool Handle(unsigned char key, int button, int state, int x, int y);
	    virtual void Update();
		virtual void Render();
		virtual string GetToken() { return m_Token; }
		void Select() { m_Selected=true; }
		void Previous(Button *s) { m_Previous=s; }
		void Next(Button *s) { m_Next=s; }
		
	private:
		string m_Name;
		float m_X,m_Y;
		bool m_Selected;
		bool m_Activated;
		Button *m_Previous;
		Button *m_Next;
		string m_Token;
	};
	
	class Text : public Widget
	{
	public:
		Text(const string &text, float x, float y);
		virtual ~Text() {}
		
		virtual bool Activated() { return false; }
		virtual bool Handle(unsigned char key, int button, int state, int x, int y) { return false; }
	    virtual void Update() {}
		virtual void Render();
		
	private:
		float m_X,m_Y;
		string m_Text;
	};
	
protected:

	string m_Title;
	GeoGame *m_Game;
	bool m_Finished;
	string m_Token;
	list<Screen::Widget*> m_Widgets;
};

}
