#include "glinc.h"
#include "key.h"
#include "bleep.h"
#include "utils.h"

using namespace geogame;

static const float MAX_GRAV_DIST=80;

Pickup::Pickup(const dVector &position, float lag) :
m_Position(position),
m_MaxSpeed(lag),
m_MaxSpeedSq(m_MaxSpeed*m_MaxSpeed),
m_FollowingMode(false),
m_FollowDistSq(1)
{
}

Pickup::~Pickup()
{
}
	
void Pickup::Update(const list<Entity*> &world, const dVector &playerpos)
{
	if (!m_FollowingMode && playerpos.distsq(m_Position)<m_FollowDistSq) 
	{
		Bleep::Get()->Play(m_CapureSound, 220+RandRange(-10,10), 1);
		m_FollowingMode=true;
		m_Flags|=ENTITYFLAG_CAUGHTKEY;
	}
	
	if (m_FollowingMode) // just follow the player around...
	{
		m_Direction+=(playerpos-m_Position)*0.01;
		m_Direction/=m_MaxSpeed*3;
	}
	else
	{
		dVector gravity;
		for (list<Entity*>::const_iterator i=world.begin(); i!=world.end(); i++)
		{
			if ((*i)->GetMass())
			{	
				dVector bet=(*i)->GetPosition()-m_Position;
				float distance = bet.mag();
				bet.normalise();
				if (distance<MAX_GRAV_DIST)
				{
					gravity+=bet*(1-(distance/MAX_GRAV_DIST))*((*i)->GetMass()*0.000001);
				}
			}
			m_Direction+=gravity;
		}
		
	}
	
	if (m_Direction.magsq()>m_MaxSpeedSq)
	{
		m_Direction.normalise();
		m_Direction*=m_MaxSpeed;
	}

	m_Position+=m_Direction;
	if (m_Position.x<-200) m_Position.x+=400;
	if (m_Position.x>200) m_Position.x-=400;
	if (m_Position.y<-200) m_Position.y+=400;
	if (m_Position.y>200) m_Position.y-=400;
	
}

void Pickup::Render()
{
}

bool Pickup::CheckCollide(const dVector &pos, float radius, bool damage)
{
	return false;
}

////////////////////////////////////////////////////////////////////////

Key::Key(const dVector &position, float lag) :
Pickup(position,lag)
{
	m_Flags|=ENTITYFLAG_KEY;
}

Key::~Key()
{
}

void Key::Update(const list<Entity*> &world, const dVector &playerpos)
{
	if (!m_FollowingMode && playerpos.distsq(m_Position)<m_FollowDistSq) 
	{
		Bleep::Get()->Play(m_CapureSound, 220+RandRange(-10,10), 1);
		m_FollowingMode=true;
		m_Flags|=ENTITYFLAG_CAUGHTKEY;
	}
	
	Pickup::Update(world,playerpos);
}

void Key::Render()
{
	if (m_FollowingMode) glColor4f(1,0,0.5,0.5);
	else glColor4f(1,0.5,0,0.5);
	glLineWidth(2);
	glPushMatrix();
	glTranslatef(m_Position.x,m_Position.y,m_Position.z);
	glRotatef(-atan(m_Direction.x/m_Direction.y)*RAD_CONV,0,0,1);	
	Primitive::Get()->Key();
	glPopMatrix();
	glLineWidth(1);
}

////////////////////////////////////////////////////////////////////////

WeaponPickup::WeaponPickup(const dVector &position, float lag) :
Pickup(position,lag)
{
	m_FollowDistSq=0.25;
	m_Flags|=ENTITYFLAG_WEAPONPICKUP;
}

WeaponPickup::~WeaponPickup()
{
}

void WeaponPickup::Update(const list<Entity*> &world, const dVector &playerpos)
{
	if (!m_FollowingMode && playerpos.distsq(m_Position)<m_FollowDistSq) 
	{
		Bleep::Get()->Play(m_CapureSound, 220+RandRange(-10,10), 1);
		m_Flags|=ENTITYFLAG_CAUGHT_WEAPONPICKUP;
		m_RemoveMe=true;
	}
	
	Pickup::Update(world,playerpos);
}

void WeaponPickup::Render()
{
	if (m_FollowingMode) glColor4f(1,0,0.5,0.5);
	else glColor4f(1,0.5,0,0.5);
	glLineWidth(2);
	glPushMatrix();
	glTranslatef(m_Position.x,m_Position.y,m_Position.z);
	glRotatef(-atan(m_Direction.x/m_Direction.y)*RAD_CONV,0,0,1);	
	Primitive::Get()->WeaponPickup();
	glPopMatrix();
	glLineWidth(1);
}

