#include "glinc.h"
#include <unistd.h>
#include "strings.h"
#include "dada.h"
#include "geogame.h"
#include "intro.h"

using namespace geogame;
using namespace std;

GeoGame *MyGeoGame;
Intro *MyGeoIntro;

unsigned int fps = (int)((1/60.0f)*1000.0f);
bool GameMode=false;

void TimerCallback(int arg)
{
	if (GameMode)
	{	
		MyGeoGame->Update();
		MyGeoGame->Render();
		if (MyGeoGame->Finished())
		{
			MyGeoIntro->Init();
			GameMode=false;
		}
	}
	else
	{
		MyGeoIntro->Update();
		MyGeoIntro->Render();
		if (MyGeoIntro->Finished()) 
		{
			MyGeoGame->Init();
			GameMode=true;
		}
	}
	
	glutSwapBuffers();
	glutTimerFunc(fps,TimerCallback,0);
}

void DisplayCallback()
{
}

void ReshapeCallback(int width, int height)
{
	MyGeoGame->Reshape(width,height);
	MyGeoIntro->Reshape(width,height);
}

void KeyboardCallback(unsigned char key,int x, int y)
{
	if (GameMode) MyGeoGame->Handle(key, 0, 0, x, y);
	else MyGeoIntro->Handle(key, 0, 0, x, y);
	
	if (key=='f') 
	{
		glutFullScreen();
	}
	
	if (key=='w') 
	{
		glutReshapeWindow(640,480);
		glutPositionWindow(100,100);
	}
}

void KeyboardUpCallback(unsigned char key,int x, int y)
{
	if (GameMode) MyGeoGame->Handle(key, 0, 1, x, y);
	else MyGeoIntro->Handle(key, 0, 1, x, y);
}

void SpecialKeyboardCallback(int key,int x, int y)
{
	if (GameMode) MyGeoGame->Handle(key, 0, 0, x, y);
	else MyGeoIntro->Handle(key, 0, 0, x, y);
}

void SpecialKeyboardUpCallback(int key,int x, int y)
{
	if (GameMode) MyGeoGame->Handle(key, 0, 1, x, y);
	else MyGeoIntro->Handle(key, 0, 1, x, y);
}

void MouseCallback(int button, int state, int x, int y)
{
	if (GameMode) MyGeoGame->Handle(0, button, state, x, y);
	else MyGeoIntro->Handle(0, button, state, x, y);
}

void MotionCallback(int x, int y)
{
	if (GameMode) MyGeoGame->Handle(0, 0, 0, x, y);
	else MyGeoIntro->Handle(0, 0, 0, x, y);
}

void IdleCallback()
{
	glutPostRedisplay();
}


int main(int argc, char *argv[])
{
	srand(time(NULL));
	InitDada();
	
	int width=640;
	int height=480;
	
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE);
	glutInitWindowSize(width, height);

	int win = glutCreateWindow("geo");
	MyGeoGame = new GeoGame(width,height,"geo");
	MyGeoIntro = new Intro(width,height,MyGeoGame);
	
	MyGeoIntro->Build("todo",TODO_SCR);
	MyGeoIntro->Build("about",ABOUT_SCR);
	MyGeoIntro->Build("info1",INFO_SCR1);
	MyGeoIntro->Build("info2",INFO_SCR2);
	MyGeoIntro->Build("info3",INFO_SCR3);
	MyGeoIntro->Build("title",TITLE_SCR);

	glutDisplayFunc(DisplayCallback);
	glutReshapeFunc(ReshapeCallback);
	glutKeyboardFunc(KeyboardCallback);
	glutSpecialFunc(SpecialKeyboardCallback); 
	glutMouseFunc(MouseCallback);
	glutMotionFunc(MotionCallback);
	glutIdleFunc(IdleCallback);
	glutKeyboardUpFunc(KeyboardUpCallback);
	glutSpecialUpFunc(SpecialKeyboardUpCallback);
	glutTimerFunc(fps,TimerCallback,0); 
	glutIgnoreKeyRepeat(1);
	
	//glutGameModeString("640x480:32");
	//glutEnterGameMode();	

	// enter the main loop
  	glutMainLoop();
	
	return 0;
}
