#include "dada.h"
#include "entity.h"

#ifndef PROJECTILE_ENTITY
#define PROJECTILE_ENTITY

namespace geogame {

class Projectile : public Entity
{
public:
	Projectile() {}
	Projectile(const dVector &pos, const dVector &vec, const dVector &inr); 
	virtual ~Projectile(); 
	virtual void Update(const list<Entity*> &world, const dVector &playerpos);
	virtual bool CheckCollide(const dVector &pos, float radius, bool damage=false) { return false; }
	virtual const dVector &GetPosition() { return m_Pos; }

	const dVector &GetVec() { return m_Vec; }
	
protected:

	dVector m_Pos;
	dVector m_Vec;
	dVector m_Inertia;
	int 	m_Age;
};

class Bullet : public Projectile
{
public:
	Bullet(const dVector &pos, const dVector &vec, const dVector &inr);
	virtual void Render();
	virtual void Update(const list<Entity*> &world, const dVector &playerpos);
};

class Missile : public Projectile
{
public:
	Missile(const dVector &pos, const dVector &vec, const dVector &inr);
	virtual void Render();
	virtual void Update(const list<Entity*> &world, const dVector &playerpos);
	void SetExplodeSound(int s) { m_ExplodeSound=s; }
private:
	bool m_Exploding;
	float m_Fade;
	int m_ExplodeSound;
};

class Rocket : public Projectile
{
public:
	Rocket(const dVector &pos, const dVector &aimpos, float speed);
	virtual void Render();
	virtual void Update(const list<Entity*> &world, const dVector &playerpos);
	virtual bool CheckCollide(const dVector &pos, float radius, bool damage=false);
	void SetExplodeSound(int s) { m_ExplodeSound=s; }
	
private:
	bool m_Exploding;
	float m_Fade;
	dVector m_AimPos;
	float m_Speed;
	int m_ExplodeSound;
};

class Mutator : public Projectile
{
public:
	Mutator(const dVector &pos, const dVector &vec, const dVector &inr);
	virtual void Render();
	virtual void Update(const list<Entity*> &world, const dVector &playerpos);
	void SetExplodeSound(int s) { m_ExplodeSound=s; }
	
private:
	bool m_Exploding;
	float m_Size;
	int m_ExplodeSound;
};
}
#endif
