#include "glinc.h"
#include "utils.h"
#include "dada.h"

using namespace geogame;

void geogame::DrawText(const string &text)
{
	glPushMatrix();
	glScalef(0.003,0.003,0.003);
	glTranslatef(0,0,-100);
	//glEnable(GL_LINE_SMOOTH);
	int count=0,line=0;
	for (unsigned int n=0; n<text.length(); n++)
	{
		glutStrokeCharacter(GLUT_STROKE_ROMAN, text[n]);
		
		if (count>80) 
		{
			line++;
			//glRasterPos3f(0.0, (-line)/10.0f, -1.1);
			count=0;
		}
		
		count++;
	}
	//glDisable(GL_LINE_SMOOTH);
	glPopMatrix();
}

//////////////////////////////////////////////////////////

Primitive *Primitive::m_Singleton=NULL;
		
Primitive::Primitive()
{
	float radius;
	
	m_Triangle = glGenLists(1);
	glNewList(m_Triangle,GL_COMPILE);
		glColor4f(1,0,0,0.5);
		radius=0.55;
		glBegin(GL_TRIANGLES);
		glVertex3f(sin(0.0*DEG_CONV)*radius,cos(0.0*DEG_CONV)*radius,0);
		glVertex3f(sin(120.0*DEG_CONV)*radius,cos(120.0*DEG_CONV)*radius,0);
		glVertex3f(sin(240.0*DEG_CONV)*radius,cos(240.0*DEG_CONV)*radius,0);
		glEnd();
	glEndList();
	
	m_Square = glGenLists(1);
	glNewList(m_Square,GL_COMPILE);
		glColor4f(0,0,1,0.5);
		radius=0.3;
		glBegin(GL_QUADS);
		glVertex3f(radius,radius,0);
		glVertex3f(-radius,radius,0);
		glVertex3f(-radius,-radius,0);
		glVertex3f(radius,-radius,0);
		glEnd();
	glEndList();

	m_ScannerKey = glGenLists(1);
	glNewList(m_ScannerKey,GL_COMPILE);
		radius=1;
		glBegin(GL_QUADS);
		glVertex3f(radius,radius,0);
		glVertex3f(-radius,radius,0);
		glVertex3f(-radius,-radius,0);
		glVertex3f(radius,-radius,0);
		glEnd();
	glEndList();


	m_Pentagon = glGenLists(1);
	glNewList(m_Pentagon,GL_COMPILE);
		glColor4f(1,0,1,0.5);
		radius=0.35;
		glBegin(GL_POLYGON);
		glVertex3f(sin(0.0*DEG_CONV)*radius,cos(0.0*DEG_CONV)*radius,0);
		glVertex3f(sin(72.0*DEG_CONV)*radius,cos(72.0*DEG_CONV)*radius,0);
		glVertex3f(sin(144.0*DEG_CONV)*radius,cos(144.0*DEG_CONV)*radius,0);
		glVertex3f(sin(216.0*DEG_CONV)*radius,cos(216.0*DEG_CONV)*radius,0);
		glVertex3f(sin(288.0*DEG_CONV)*radius,cos(288.0*DEG_CONV)*radius,0);
		glEnd();
	glEndList();
	
	m_Particle = glGenLists(1);
	glNewList(m_Particle,GL_COMPILE);
		radius=0.35;
		glBegin(GL_POLYGON);
		glVertex3f(sin(0.0*DEG_CONV)*radius,cos(0.0*DEG_CONV)*radius,0);
		glVertex3f(sin(72.0*DEG_CONV)*radius,cos(72.0*DEG_CONV)*radius,0);
		glVertex3f(sin(144.0*DEG_CONV)*radius,cos(144.0*DEG_CONV)*radius,0);
		glVertex3f(sin(216.0*DEG_CONV)*radius,cos(216.0*DEG_CONV)*radius,0);
		glVertex3f(sin(288.0*DEG_CONV)*radius,cos(288.0*DEG_CONV)*radius,0);
		glEnd();
	glEndList();
	
	m_Key = glGenLists(1);
	glNewList(m_Key,GL_COMPILE);
		radius=1;
		glBegin(GL_LINE_LOOP);
		glVertex3f(sin(0.0*DEG_CONV)*radius,cos(0.0*DEG_CONV)*radius,0);
		glVertex3f(sin(120.0*DEG_CONV)*radius,cos(120.0*DEG_CONV)*radius,0);
		glVertex3f(sin(240.0*DEG_CONV)*radius,cos(240.0*DEG_CONV)*radius,0);
		glEnd();
	glEndList();
	
	m_Explosion = glGenLists(1);
	glNewList(m_Explosion,GL_COMPILE);
		radius=1;
		glBegin(GL_POLYGON);
		for(int s=0; s<=12; s++)
		{
			glVertex3f(sin((360/12.0f)*s*DEG_CONV),
			           cos((360/12.0f)*s*DEG_CONV),0);
		}
		glEnd();
		glPopMatrix();
	glEndList();	
	
	m_WeaponPickup = glGenLists(1);
	glNewList(m_WeaponPickup,GL_COMPILE);
		radius=0.35;
		glBegin(GL_LINE_LOOP);
		glVertex3f(sin(0.0*DEG_CONV*2)*radius,cos(0.0*DEG_CONV*2)*radius,0);
		glVertex3f(sin(72.0*DEG_CONV*2)*radius,cos(72.0*DEG_CONV*2)*radius,0);
		glVertex3f(sin(144.0*DEG_CONV*2)*radius,cos(144.0*DEG_CONV*2)*radius,0);
		glVertex3f(sin(216.0*DEG_CONV*2)*radius,cos(216.0*DEG_CONV*2)*radius,0);
		glVertex3f(sin(288.0*DEG_CONV*2)*radius,cos(288.0*DEG_CONV*2)*radius,0);
		glEnd();
	glEndList();

}

Primitive::~Primitive()
{
}

Primitive *Primitive::Get()
{
	if (!m_Singleton) m_Singleton=new Primitive;
	return m_Singleton;
}

